@@ -8,7 +8,6 @@ module Prism
   class FixturesTest < TestCase
     except = []
 
-
     if RUBY_VERSION < "3.3.0"
       # Ruby < 3.3.0 cannot parse heredocs where there are leading whitespace
       # characters in the heredoc start.
@@ -25,6 +24,12 @@ module Prism
       except << "whitequark/ruby_bug_19281.txt"
     end
 
+    # Leaving these out until they are supported by parse.y.
+    except << "leading_logical.txt"
+    except << "endless_methods_command_call.txt"
+    # https://bugs.ruby-lang.org/issues/21168#note-5
+    except << "command_method_call_2.txt"
+
     Fixture.each(except: except) do |fixture|
       define_method(fixture.test_name) { assert_valid_syntax(fixture.read) }
     end
